@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm, Török Edvin
@@ -33,6 +33,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <dirent.h>
+#include <fcntl.h>
 #ifndef _WIN32
 #include <sys/time.h>
 #include <sys/wait.h>
@@ -50,7 +51,7 @@
 #include "others.h"
 #include "scanners.h"
 
-// shared
+// common
 #include "idmef_logging.h"
 #include "optparser.h"
 #include "output.h"
@@ -146,13 +147,13 @@ cl_error_t scan_callback(STATBUF *sb, char *filename, const char *msg, enum cli_
 
     if (NULL != filename) {
         if (CL_SUCCESS != cli_realpath((const char *)filename, &real_filename)) {
-            conn_reply_errno(scandata->conn, msg, "real-path check failed:");
-            logg("^Failed to determine real path for: %s\n", filename);
-            scandata->errors++;
-            return CL_SUCCESS;
+            conn_reply_errno(scandata->conn, msg, "File path check failure:");
+            logg("^File path check failure for: %s\n", filename);
+            logg("*Quarantine of the file may fail if file path contains symlinks.\n");
+        } else {
+            free(filename);
+            filename = real_filename;
         }
-        free(filename);
-        filename = real_filename;
     }
 
     /* detect disconnected socket,
@@ -180,25 +181,30 @@ cl_error_t scan_callback(STATBUF *sb, char *filename, const char *msg, enum cli_
             else
                 logg("!Memory allocation failed during cli_ftw()\n");
             scandata->errors++;
+            free(filename);
             return CL_EMEM;
         case error_stat:
-            conn_reply_errno(scandata->conn, msg, "lstat() failed:");
-            logg("^lstat() failed on: %s\n", msg);
+            conn_reply_errno(scandata->conn, msg, "File path check failure:");
+            logg("^File path check failure on: %s\n", msg);
             scandata->errors++;
+            free(filename);
             return CL_SUCCESS;
         case warning_skipped_dir:
-            logg("^Directory recursion limit reached, skipping %s\n",
-                 msg);
+            logg("^Directory recursion limit reached, skipping %s\n", msg);
+            free(filename);
             return CL_SUCCESS;
         case warning_skipped_link:
             logg("$Skipping symlink: %s\n", msg);
+            free(filename);
             return CL_SUCCESS;
         case warning_skipped_special:
             if (msg == scandata->toplevel_path)
                 conn_reply(scandata->conn, msg, "Not supported file type", "ERROR");
             logg("*Not supported file type: %s\n", msg);
+            free(filename);
             return CL_SUCCESS;
         case visit_directory_toplev:
+            free(filename);
             return CL_SUCCESS;
         case visit_file:
             break;
@@ -257,17 +263,6 @@ cl_error_t scan_callback(STATBUF *sb, char *filename, const char *msg, enum cli_
         return CL_SUCCESS;
     }
 
-    if (access(filename, R_OK)) {
-        if (conn_reply(scandata->conn, filename, "Access denied.", "ERROR") == -1) {
-            free(filename);
-            return CL_ETIMEOUT;
-        }
-        logg("*Access denied: %s\n", filename);
-        scandata->errors++;
-        free(filename);
-        return CL_SUCCESS;
-    }
-
     thrmgr_setactivetask(filename, NULL);
     context.filename = filename;
     context.virsize  = 0;
@@ -286,6 +281,17 @@ cl_error_t scan_callback(STATBUF *sb, char *filename, const char *msg, enum cli_
         ret = CL_EMEM;
     }
 
+    if (ret == CL_EACCES) {
+        if (conn_reply(scandata->conn, filename, "Access denied.", "ERROR") == -1) {
+            free(filename);
+            return CL_ETIMEOUT;
+        }
+        logg("*Access denied: %s\n", filename);
+        scandata->errors++;
+        free(filename);
+        return CL_SUCCESS;
+    }
+
     if (ret == CL_VIRUS) {
 
         if (scandata->options->general & CL_SCAN_GENERAL_ALLMATCHES || (scandata->infected && scandata->options->general & CL_SCAN_GENERAL_HEURISTIC_PRECEDENCE)) {
@@ -295,6 +301,7 @@ cl_error_t scan_callback(STATBUF *sb, char *filename, const char *msg, enum cli_
             virusaction(filename, virname, scandata->opts);
         } else {
             scandata->infected++;
+            virusaction(filename, virname, scandata->opts);
             if (conn_reply_virus(scandata->conn, filename, virname) == -1) {
                 free(filename);
                 return CL_ETIMEOUT;
@@ -307,7 +314,6 @@ cl_error_t scan_callback(STATBUF *sb, char *filename, const char *msg, enum cli_
                 logg("~%s: %s(%s:%llu) FOUND\n", filename, virname, context.virhash, context.virsize);
             else
                 logg("~%s: %s FOUND\n", filename, virname);
-            virusaction(filename, virname, scandata->opts);
         }
     } else if (ret != CL_CLEAN) {
         scandata->errors++;
@@ -364,7 +370,7 @@ int scan_pathchk(const char *path, struct cli_ftw_cbdata *data)
     return 0;
 }
 
-int scanfd(
+cl_error_t scanfd(
     const client_conn_t *conn,
     unsigned long int *scanned,
     const struct cl_engine *engine,
@@ -373,13 +379,17 @@ int scanfd(
     int odesc,
     int stream)
 {
-    int ret, fd = conn->scanfd;
+    cl_error_t ret      = -1;
+    int fd              = conn->scanfd;
     const char *virname = NULL;
     STATBUF statbuf;
     struct cb_context context;
     char fdstr[32];
     const char *reply_fdstr;
 
+    char *filepath     = NULL;
+    char *log_filename = fdstr;
+
     UNUSEDPARAM(odesc);
 
     if (stream) {
@@ -396,41 +406,60 @@ int scanfd(
     }
     if (FSTAT(fd, &statbuf) == -1 || !S_ISREG(statbuf.st_mode)) {
         logg("%s: Not a regular file. ERROR\n", fdstr);
-        if (conn_reply(conn, reply_fdstr, "Not a regular file", "ERROR") == -1)
-            return CL_ETIMEOUT;
-        return -1;
+        if (conn_reply(conn, reply_fdstr, "Not a regular file", "ERROR") == -1) {
+            ret = CL_ETIMEOUT;
+            goto done;
+        }
+        ret = CL_BREAK;
+        goto done;
+    }
+
+    /* Try and get the real filename, for logging purposes */
+    if (!stream) {
+        if (CL_SUCCESS != cli_get_filepath_from_filedesc(fd, &filepath)) {
+            logg("*%s: Unable to determine the filepath given the file descriptor.\n", fdstr);
+        } else {
+            log_filename = filepath;
+        }
     }
 
     thrmgr_setactivetask(fdstr, NULL);
     context.filename = fdstr;
     context.virsize  = 0;
     context.scandata = NULL;
-    ret              = cl_scandesc_callback(fd, conn->filename, &virname, scanned, engine, options, &context);
+    ret              = cl_scandesc_callback(fd, log_filename, &virname, scanned, engine, options, &context);
     thrmgr_setactivetask(NULL, NULL);
 
     if (thrmgr_group_need_terminate(conn->group)) {
         logg("*Client disconnected while scanjob was active\n");
-        return ret == CL_ETIMEOUT ? ret : CL_BREAK;
+        ret = ret == CL_ETIMEOUT ? ret : CL_BREAK;
+        goto done;
     }
 
     if (ret == CL_VIRUS) {
+        virusaction(log_filename, virname, opts);
         if (conn_reply_virus(conn, reply_fdstr, virname) == -1)
             ret = CL_ETIMEOUT;
         if (context.virsize && optget(opts, "ExtendedDetectionInfo")->enabled)
-            logg("%s: %s(%s:%llu) FOUND\n", fdstr, virname, context.virhash, context.virsize);
+            logg("%s: %s(%s:%llu) FOUND\n", log_filename, virname, context.virhash, context.virsize);
         else
-            logg("%s: %s FOUND\n", fdstr, virname);
-        virusaction(reply_fdstr, virname, opts);
+            logg("%s: %s FOUND\n", log_filename, virname);
     } else if (ret != CL_CLEAN) {
         if (conn_reply(conn, reply_fdstr, cl_strerror(ret), "ERROR") == -1)
             ret = CL_ETIMEOUT;
-        logg("%s: %s ERROR\n", fdstr, cl_strerror(ret));
+        logg("%s: %s ERROR\n", log_filename, cl_strerror(ret));
     } else {
         if (conn_reply_single(conn, reply_fdstr, "OK") == CL_ETIMEOUT)
             ret = CL_ETIMEOUT;
         if (logok)
-            logg("%s: OK\n", fdstr);
+            logg("%s: OK\n", log_filename);
     }
+
+done:
+    if (NULL != filepath) {
+        free(filepath);
+    }
+
     return ret;
 }
 
