@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm, Török Edvin
@@ -40,7 +40,7 @@
 // libclamav
 #include "clamav.h"
 
-// shared
+// common
 #include "optparser.h"
 #include "output.h"
 #include "misc.h"
@@ -126,6 +126,7 @@ int tcpserver(int **lsockets, unsigned int *nlsockets, char *ipaddr, const struc
             for (i = 0; i < *nlsockets; i++)
                 close(sockets[i]);
 
+            freeaddrinfo(info);
             return -1;
         }
         sockets = t;
