@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Trog, Török Edvin
@@ -30,14 +30,17 @@
 #include <errno.h>
 #include <string.h>
 
-#include "shared/output.h"
-
-#include "libclamav/clamav.h"
-#include "thrmgr.h"
+// libclamav
+#include "clamav.h"
 #include "others.h"
 #include "mpool.h"
+
+// common
+#include "output.h"
+
+#include "thrmgr.h"
+#include "clamd_others.h"
 #include "server.h"
-#include "libclamav/others.h"
 
 #ifdef HAVE_MALLINFO
 #include <malloc.h>
@@ -359,6 +362,45 @@ void thrmgr_destroy(threadpool_t *threadpool)
     return;
 }
 
+void thrmgr_wait_for_threads(threadpool_t *threadpool)
+{
+    if (!threadpool) {
+        return;
+    }
+    if (pthread_mutex_lock(&threadpool->pool_mutex) != 0) {
+        logg("!Mutex lock failed\n");
+        exit(-1);
+    }
+    if (threadpool->state != POOL_VALID) {
+        if (pthread_mutex_unlock(&threadpool->pool_mutex) != 0) {
+            logg("!Mutex unlock failed\n");
+            exit(-1);
+        }
+        return;
+    }
+
+    /* wait for threads to exit */
+    if (threadpool->thr_alive > 0) {
+        if (pthread_cond_broadcast(&(threadpool->pool_cond)) != 0) {
+            pthread_mutex_unlock(&threadpool->pool_mutex);
+            return;
+        }
+    }
+    while (threadpool->thr_alive > 0) {
+        if (pthread_cond_wait(&threadpool->pool_cond, &threadpool->pool_mutex) != 0) {
+            pthread_mutex_unlock(&threadpool->pool_mutex);
+            return;
+        }
+    }
+
+    /* Ok threads all exited, we can release the lock */
+    if (pthread_mutex_unlock(&threadpool->pool_mutex) != 0) {
+        logg("!Mutex unlock failed\n");
+        exit(-1);
+    }
+    return;
+}
+
 threadpool_t *thrmgr_new(int max_threads, int idle_timeout, int max_queue, void (*handler)(void *))
 {
     threadpool_t *threadpool;
