@@ -128,6 +128,7 @@ const char *autoit_functions[] = {
     "DRIVESPACEFREE",
     "DRIVESPACETOTAL",
     "DRIVESTATUS",
+    "DUMMYSPEEDTEST",
     "ENVGET",
     "ENVSET",
     "ENVUPDATE",
@@ -135,7 +136,6 @@ const char *autoit_functions[] = {
     "EXECUTE",
     "EXP",
     "FILECHANGEDIR",
-    "UNKNOWN_89",
     "FILECLOSE",
     "FILECOPY",
     "FILECREATENTFSLINK",
@@ -275,17 +275,17 @@ const char *autoit_functions[] = {
     "ISHWND",
     "ISINT",
     "ISKEYWORD",
-    "UNKNOWN_229",
+    "ISMAP",
     "ISNUMBER",
     "ISOBJ",
     "ISPTR",
     "ISSTRING",
     "LOG",
+    "MAPAPPEND",
+    "MAPEXISTS",
+    "MAPKEYS",
+    "MAPREMOVE",
     "MEMGETSTATS",
-    "UNKNOWN_235",
-    "UNKNOWN_236",
-    "UNKNOWN_237",
-    "UNKNOWN_238",
     "MOD",
     "MOUSECLICK",
     "MOUSECLICKDRAG",
@@ -393,8 +393,8 @@ const char *autoit_functions[] = {
     "TCPNAMETOIP",
     "TCPRECV",
     "TCPSEND",
-    "TCPSHUTDOWN,",
-    "TCPSTARTUP,",
+    "TCPSHUTDOWN",
+    "TCPSTARTUP",
     "TIMERDIFF",
     "TIMERINIT",
     "TOOLTIP",
@@ -421,8 +421,8 @@ const char *autoit_functions[] = {
     "UDPOPEN",
     "UDPRECV",
     "UDPSEND",
-    "UNKNOWN_375",
-    "UNKNOWN_376",
+    "UDPSHUTDOWN",
+    "UDPSTARTUP",
     "VARGETTYPE",
     "WINACTIVATE",
     "WINACTIVE",
@@ -454,7 +454,7 @@ const char *autoit_functions[] = {
     "WINWAITNOTACTIVE"};
 
 const char *autoit_keywords[] = {
-    "UNKNOWN_0",
+    "UNKNOWN_0", // "".
     "AND",
     "OR",
     "NOT",
@@ -497,7 +497,7 @@ const char *autoit_keywords[] = {
     "FALSE",
     "DEFAULT",
     "NULL",
-    "UNKNOWN_43",
+    "VOLATILE",
     "ENUM",
 };
 
@@ -646,7 +646,7 @@ static int ea05(cli_ctx *ctx, const uint8_t *base, char *tmpd)
     unsigned int files = 0;
     char tempfile[1024];
     struct UNP UNP;
-    fmap_t *map = ctx->fmap;
+    fmap_t *map = *ctx->fmap;
 
     if (!fmap_need_ptr_once(map, base, 16))
         return CL_CLEAN;
@@ -969,10 +969,10 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
     int i, ret, det = 0;
     unsigned int files = 0;
     char tempfile[1024];
-    const char prefixes[] = {'\0', '\0', '@', '$', '\0', '.', '"', '#'};
+    const char prefixes[] = {'\0', '\0', '@', '$', '\0', '.', '"', '\0'};
     const char *opers[]   = {",", "=", ">", "<", "<>", ">=", "<=", "(", ")", "+", "-", "/", "*", "&", "[", "]", "==", "^", "+=", "-=", "/=", "*=", "&=", "?", ":"};
     struct UNP UNP;
-    fmap_t *map = ctx->fmap;
+    fmap_t *map = *ctx->fmap;
 
     /* Useless due to a bug in CRC calculation - LMAO!!1 */
     /*   if (cli_readn(desc, buf, 24)!=24) */
@@ -1372,6 +1372,7 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
                             UNP.cur_input += dchars;
                         }
                         if (op == 0x36)
+                            // TODO: Mask possible double quotes inside the string: >Say:"Hi "<  ==> >"Say:""Hi"" "<
                             buf[UNP.cur_output++] = '"';
                         if (op != 0x34)
                             buf[UNP.cur_output++] = ' ';
@@ -1411,6 +1412,13 @@ static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd)
                             }
                             buf = newout;
                         }
+
+                        //TODO: Fix Autoit plus bug
+                        // if (op == 0x49) /* + */ and next op ==0x05 /*int32*/ and that int32 is negative...
+                        // skip next line (and don't add "+")
+                        // Background: "$a= (-4)" gets incorrect compiled. Decompiled it will be get "$A= (+ -4)"
+                        // That doesn't effects the interpreter however when recompiling decompiled output that will result in a syntax error
+
                         UNP.cur_output += snprintf((char *)&buf[UNP.cur_output], 4, "%s ", opers[op - 0x40]);
                         break;
 
@@ -1491,7 +1499,7 @@ int cli_scanautoit(cli_ctx *ctx, off_t offset)
     const uint8_t *version;
     int r;
     char *tmpd;
-    fmap_t *map = ctx->fmap;
+    fmap_t *map = *ctx->fmap;
 
     cli_dbgmsg("in scanautoit()\n");
 
