@@ -1,7 +1,7 @@
 /*
  *  Compile LLVM bytecode to ClamAV bytecode.
  *
- *  Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2010-2013 Sourcefire, Inc.
  *
  *  Authors: Török Edvin
@@ -20,9 +20,10 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *  MA 02110-1301, USA.
  */
-namespace llvm {
-    class Value;
+namespace llvm
+{
+class Value;
 }
 // match the prototype used in the bytecode compiler
-void printValue(llvm::Value *V, bool a=false, bool b=false);
-void printLocation(llvm::Instruction *I, bool a=false, bool b=false);
+void printValue(llvm::Value *V, bool a = false, bool b = false);
+void printLocation(llvm::Instruction *I, bool a = false, bool b = false);
