@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
@@ -125,6 +125,9 @@ struct cli_dconf {
 #define OTHER_CONF_PDFNAMEOBJ   0x100
 #define OTHER_CONF_PRTNINTXN    0x200
 #define OTHER_CONF_LZW          0x400
+#define OTHER_CONF_PNG          0x800
+#define OTHER_CONF_GIF          0x1000
+#define OTHER_CONF_TIFF         0x2000
 
 /* Phishing flags */
 #define PHISHING_CONF_ENGINE  0x1
