@@ -35,21 +35,7 @@ struct cli_target_info {
     int status; /* 0 == not initialised, 1 == initialised OK, -1 == error */
 };
 
-/**
- * Initialize a struct cli_target_info so that it's ready to have its exeinfo
- * populated by the call to cli_targetinfo and/or destroyed by
- * cli_targetinfo_destroy.
- *
- * @param info a pointer to the struct cli_target_info to initialize
- */
 void cli_targetinfo_init(struct cli_target_info *info);
-
-/**
- * Free resources associated with a struct cli_target_info initialized
- * via cli_targetinfo_init
- *
- * @param info a pointer to the struct cli_target_info to destroy
- */
 void cli_targetinfo_destroy(struct cli_target_info *info);
 
 #include "matcher-ac.h"
@@ -243,12 +229,12 @@ static const struct cli_mtarget cli_mtargets[CLI_MTARGETS] = {
  * This function does not perform file type magic identification and does not use
  * the file format scanners.
  *
- * @param buffer    The buffer to be matched.
- * @param length    The length of the buffer or amount of bytets to match.
- * @param offset    Offset into the buffer from which to start matching.
- * @param ctx       The scanning context.
- * @param ftype     If specified, may limit signature matching trie by target type corresponding with the specified CL_TYPE
- * @param acdata    [in/out] A list of pattern maching data structs to contain match results, one for each pattern matching trie.
+ * @param buffer            The buffer to be matched.
+ * @param length            The length of the buffer or amount of bytets to match.
+ * @param offset            Offset into the buffer from which to start matching.
+ * @param ctx               The scanning context.
+ * @param ftype             If specified, may limit signature matching trie by target type corresponding with the specified CL_TYPE
+ * @param[in,out] acdata    A list of pattern maching data structs to contain match results, one for each pattern matching trie.
  * @return cl_error_t
  */
 cl_error_t cli_scan_buff(const unsigned char *buffer, uint32_t length, uint32_t offset, cli_ctx *ctx, cli_file_t ftype, struct cli_ac_data **acdata);
@@ -261,14 +247,14 @@ cl_error_t cli_scan_buff(const unsigned char *buffer, uint32_t length, uint32_t
  *
  * This function uses the newer cli_scan_fmap() scanning API.
  *
- * @param desc      File descriptor to be used for input
- * @param ctx       The scanning context.
- * @param ftype     If specified, may limit signature matching trie by target type corresponding with the specified CL_TYPE
- * @param ftonly    Boolean indicating if the scan is for file-type detection only.
- * @param ftoffset  [out] A list of file type signature matches with their corresponding offsets.
- * @param acmode    Use AC_SCAN_VIR and AC_SCAN_FT to set scanning modes.
- * @param acres     [out] A list of cli_ac_result AC pattern matching results.
- * @param name      (optional) Original name of the file (to set fmap name metadata)
+ * @param desc          File descriptor to be used for input
+ * @param ctx           The scanning context.
+ * @param ftype         If specified, may limit signature matching trie by target type corresponding with the specified CL_TYPE
+ * @param ftonly        Boolean indicating if the scan is for file-type detection only.
+ * @param[out] ftoffset A list of file type signature matches with their corresponding offsets.
+ * @param acmode        Use AC_SCAN_VIR and AC_SCAN_FT to set scanning modes.
+ * @param[out] acres    A list of cli_ac_result AC pattern matching results.
+ * @param name          (optional) Original name of the file (to set fmap name metadata)
  * @return cl_error_t
  */
 cl_error_t cli_scan_desc(int desc, cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli_matched_type **ftoffset, unsigned int acmode, struct cli_ac_result **acres, const char *name);
@@ -278,13 +264,13 @@ cl_error_t cli_scan_desc(int desc, cli_ctx *ctx, cli_file_t ftype, uint8_t ftonl
  *
  * This API will invoke cli_exp_eval() for you.
  *
- * @param ctx       The scanning context.
- * @param ftype     If specified, may limit signature matching trie by target type corresponding with the specified CL_TYPE
- * @param ftonly    Boolean indicating if the scan is for file-type detection only.
- * @param ftoffset  [out] A list of file type signature matches with their corresponding offsets.
- * @param acmode    Use AC_SCAN_VIR and AC_SCAN_FT to set scanning modes.
- * @param acres     [out] A list of cli_ac_result AC pattern matching results.
- * @param refhash   MD5 hash of the current file, used to save time creating hashes and to limit scan recursion for the HandlerType logical signature FTM feature.
+ * @param ctx           The scanning context.
+ * @param ftype         If specified, may limit signature matching trie by target type corresponding with the specified CL_TYPE
+ * @param ftonly        Boolean indicating if the scan is for file-type detection only.
+ * @param[out] ftoffset A list of file type signature matches with their corresponding offsets.
+ * @param acmode        Use AC_SCAN_VIR and AC_SCAN_FT to set scanning modes.
+ * @param[out] acres    A list of cli_ac_result AC pattern matching results.
+ * @param refhash       MD5 hash of the current file, used to save time creating hashes and to limit scan recursion for the HandlerType logical signature FTM feature.
  * @return cl_error_t
  */
 cl_error_t cli_scan_fmap(cli_ctx *ctx, cli_file_t ftype, uint8_t ftonly, struct cli_matched_type **ftoffset, unsigned int acmode, struct cli_ac_result **acres, unsigned char *refhash);
@@ -304,34 +290,11 @@ cl_error_t cli_exp_eval(cli_ctx *ctx, struct cli_matcher *root, struct cli_ac_da
 
 cl_error_t cli_caloff(const char *offstr, const struct cli_target_info *info, unsigned int target, uint32_t *offdata, uint32_t *offset_min, uint32_t *offset_max);
 
-/**
- * @brief Determine if an alert is a known false positive, using each fmap in the the ctx->container stack to check MD5, SHA1, and SHA256 hashes.
- *
- * @param ctx           The scanning context.
- * @param vname         (Optional) The name of the signature alert.
- * @return cl_error_t   CL_CLEAN If an allow-list hash matches with one of the fmap hashes in the scan recursion stack.
- *                      CL_VIRUS If no allow-list hash matches.
- */
-cl_error_t cli_check_fp(cli_ctx *ctx, const char *vname);
+cl_error_t cli_checkfp(cli_ctx *ctx);
+cl_error_t cli_checkfp_virus(cli_ctx *ctx, const char *vname, uint32_t recursion_cnt);
 
 cl_error_t cli_matchmeta(cli_ctx *ctx, const char *fname, size_t fsizec, size_t fsizer, int encrypted, unsigned int filepos, int res1, void *res2);
 
-/** Parse the executable headers and, if successful, populate exeinfo
- *
- * If target refers to a supported executable file type, the exe header
- * will be parsed and, if successful, info->status will be set to 1.
- * If parsing the exe header fails, info->status will be set to -1.
- * The caller MUST destroy info via a call to cli_targetinfo_destroy
- * regardless of what info->status is set to.
- *
- * @param info A structure to populate with info from the exe header. This
- *             MUST be initialized via cli_targetinfo_init prior to calling
- * @param target the target executable file type. Possible values are:
- *               - 1 - PE32 / PE32+
- *               - 6 - ELF
- *               - 9 - MachO
- * @param ctx The current scan context
- */
-void cli_targetinfo(struct cli_target_info *info, unsigned int target, cli_ctx *ctx);
+void cli_targetinfo(struct cli_target_info *info, unsigned int target, fmap_t *map);
 
 #endif
