@@ -38,7 +38,6 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <stdbool.h>
 
 #ifdef HAVE_JSON
 #include <json.h>
@@ -73,7 +72,7 @@
  * in re-enabling affected modules.
  */
 
-#define CL_FLEVEL 125
+#define CL_FLEVEL 140
 #define CL_FLEVEL_DCONF CL_FLEVEL
 #define CL_FLEVEL_SIGTOOL CL_FLEVEL
 
@@ -98,31 +97,13 @@ extern uint8_t cli_always_gen_section_hash;
      (size_t)(sb) < (size_t)(bb) + (size_t)(bb_size))
 
 /*
- * CLI_ISCONTAINED_0_TO(bb_size, sb, sb_size) checks if sb (small offset) is
- * within bb (big offset) where the big offset always starts at 0.
- *
- * bb and sb are offsets for the main buffer and the
- * sub-buffer respectively, and bb_size and sb_size are their sizes
- *
- * The macro can be used to protect against wraps.
- *
- * CLI_ISCONTAINED_0_TO is the same as CLI_ISCONTAINED except that `bb` is gone
- * and assumed ot be zero.
- */
-#define CLI_ISCONTAINED_0_TO(bb_size, sb, sb_size)            \
-    ((size_t)(bb_size) > 0 && (size_t)(sb_size) > 0 &&        \
-     (size_t)(sb_size) <= (size_t)(bb_size) &&                \
-     (size_t)(sb) + (size_t)(sb_size) <= (size_t)(bb_size) && \
-     (size_t)(sb) < (size_t)(bb_size))
-
-/*
- * CLI_ISCONTAINED_2(bb, bb_size, sb, sb_size) checks if sb (small buffer) is
+ * CLI_ISCONTAINED2(bb, bb_size, sb, sb_size) checks if sb (small buffer) is
  * within bb (big buffer).
  *
- * CLI_ISCONTAINED_2 is the same as CLI_ISCONTAINED except that it allows for
+ * CLI_ISCONTAINED2 is the same as CLI_ISCONTAINED except that it allows for
  * small-buffers with sb_size == 0.
  */
-#define CLI_ISCONTAINED_2(bb, bb_size, sb, sb_size)                          \
+#define CLI_ISCONTAINED2(bb, bb_size, sb, sb_size)                           \
     ((size_t)(bb_size) > 0 &&                                                \
      (size_t)(sb_size) <= (size_t)(bb_size) &&                               \
      (size_t)(sb) >= (size_t)(bb) &&                                         \
@@ -159,19 +140,16 @@ typedef struct bitset_tag {
     unsigned long length;
 } bitset_t;
 
-typedef struct recursion_level_tag {
+typedef struct cli_ctx_container_tag {
     cli_file_t type;
     size_t size;
-    cl_fmap_t *fmap;                      /* The fmap for this layer. This used to be in an array in the ctx. */
-    uint32_t recursion_level_buffer;      /* Which buffer layer in scan recursion. */
-    uint32_t recursion_level_buffer_fmap; /* Which fmap layer in this buffer. */
-    bool is_normalized_layer;             /* Indicates that the layer should be skipped when checking container and intermediate types. */
-} recursion_level_t;
-// #define CONTAINER_FLAG_VALID 0x01
+    unsigned char flag;
+} cli_ctx_container;
+#define CONTAINER_FLAG_VALID 0x01
 
 /* internal clamav context */
 typedef struct cli_ctx_tag {
-    char *target_filepath;    /**< (optional) The filepath of the original scan target. */
+    char *target_filepath;    /**< (optional) The filepath of the original scan target */
     const char *sub_filepath; /**< (optional) The filepath of the current file being parsed. May be a temp file. */
     char *sub_tmpdir;         /**< The directory to store tmp files at this recursion depth. */
     const char **virname;
@@ -181,17 +159,15 @@ typedef struct cli_ctx_tag {
     const struct cl_engine *engine;
     unsigned long scansize;
     struct cl_scan_options *options;
+    unsigned int recursion;
     unsigned int scannedfiles;
     unsigned int found_possibly_unwanted;
     unsigned int corrupted_input;
     unsigned int img_validate;
-    recursion_level_t *recursion_stack; /* Array of recursion levels used as a stack. */
-    uint32_t recursion_stack_size;      /* stack size must == engine->max_recursion_level */
-    uint32_t recursion_level;           /* Index into recursion_stack; current fmap recursion level from start of scan. */
-    fmap_t *fmap;                       /* Pointer to current fmap in recursion_stack, varies with recursion depth. For convenience. */
-    bool next_layer_is_normalized;      /* Indicate that the next fmap pushed to the stack is normalized and should be ignored when checking container/intermediate types */
+    cli_ctx_container *containers; /* set container type after recurse */
     unsigned char handlertype_hash[16];
     struct cli_dconf *dconf;
+    fmap_t **fmap; /* pointer to current fmap in an allocated array, incremented with recursion depth */
     bitset_t *hook_lsig_matches;
     void *cb_ctx;
     cli_events_t *perf;
@@ -203,8 +179,7 @@ typedef struct cli_ctx_tag {
     struct json_object *wrkproperty;
 #endif
     struct timeval time_limit;
-    bool limit_exceeded; /* To guard against alerting on limits exceeded more than once, or storing that in the JSON metadata more than once. */
-    bool abort_scan;     /* So we can guarantee a scan is aborted, even if CL_ETIMEOUT/etc. status is lost in the scan recursion stack. */
+    int limit_exceeded;
 } cli_ctx;
 
 #define STATS_ANON_UUID "5b585e8f-3be5-11e3-bf0b-18037319526c"
@@ -310,15 +285,15 @@ struct cl_engine {
     uint64_t engine_options;
 
     /* Limits */
-    uint32_t maxscantime;         /* Time limit (in milliseconds) */
-    uint64_t maxscansize;         /* during the scanning of archives this size
+    uint32_t maxscantime; /* Time limit (in milliseconds) */
+    uint64_t maxscansize; /* during the scanning of archives this size
 				           * will never be exceeded
 				           */
-    uint64_t maxfilesize;         /* compressed files will only be decompressed
+    uint64_t maxfilesize; /* compressed files will only be decompressed
 				           * and scanned up to this size
 				           */
-    uint32_t max_recursion_level; /* maximum recursion level for archives */
-    uint32_t maxfiles;            /* maximum number of files to be scanned
+    uint32_t maxreclevel; /* maximum recursion level for archives */
+    uint32_t maxfiles;    /* maximum number of files to be scanned
 				           * within a single archive
 				           */
     /* This is for structured data detection.  You can set the minimum
@@ -337,15 +312,15 @@ struct cl_engine {
     struct cli_matcher *hm_mdb;
     /* hash matcher for MD5 sigs for PE import tables */
     struct cli_matcher *hm_imp;
-    /* hash matcher for whitelist db */
+    /* hash matcher for allow list db */
     struct cli_matcher *hm_fp;
 
     /* Container metadata */
     struct cli_cdb *cdb;
 
     /* Phishing .pdb and .wdb databases*/
-    struct regex_matcher *whitelist_matcher;
-    struct regex_matcher *domainlist_matcher;
+    struct regex_matcher *allow_list_matcher;
+    struct regex_matcher *domain_list_matcher;
     struct phishcheck *phishcheck;
 
     /* Dynamic configuration */
@@ -378,6 +353,9 @@ struct cl_engine {
     /* Database information from .info files */
     struct cli_dbinfo *dbinfo;
 
+    /* Signature counting, for progress callbacks */
+    size_t num_total_signatures;
+
     /* Used for memory pools */
     mpool_t *mempool;
 
@@ -394,6 +372,12 @@ struct cl_engine {
     clcb_hash cb_hash;
     clcb_meta cb_meta;
     clcb_file_props cb_file_props;
+    clcb_progress cb_sigload_progress;
+    void *cb_sigload_progress_ctx;
+    clcb_progress cb_engine_compile_progress;
+    void *cb_engine_compile_progress_ctx;
+    clcb_progress cb_engine_free_progress;
+    void *cb_engine_free_progress_ctx;
 
     /* Used for bytecode */
     struct cli_all_bc bcs;
@@ -453,7 +437,7 @@ struct cl_settings {
     uint32_t maxscantime;
     uint64_t maxscansize;
     uint64_t maxfilesize;
-    uint32_t max_recursion_level;
+    uint32_t maxreclevel;
     uint32_t maxfiles;
     uint32_t min_cc_count;
     uint32_t min_ssn_count;
@@ -474,6 +458,12 @@ struct cl_settings {
     clcb_hash cb_hash;
     clcb_meta cb_meta;
     clcb_file_props cb_file_props;
+    clcb_progress cb_sigload_progress;
+    void *cb_sigload_progress_ctx;
+    clcb_progress cb_engine_compile_progress;
+    void *cb_engine_compile_progress_ctx;
+    clcb_progress cb_engine_free_progress;
+    void *cb_engine_free_progress_ctx;
 
     /* Engine max settings */
     uint64_t maxembeddedpe;      /* max size to scan MSEXE for PE */
@@ -664,8 +654,8 @@ static inline void cli_writeint32(void *offset, uint32_t value)
 /**
  * @brief Append an alert.
  *
- * An FP-check will verify that the file is not whitelisted.
- * The whitelist check does not happen before the scan because file whitelisting
+ * An FP-check will verify that the file is not allowed.
+ * The allow list check does not happen before the scan because allowing files
  * is so infrequent that such action would be detrimental to performance.
  *
  * TODO: Replace implementation with severity scale, and severity threshold
@@ -701,74 +691,10 @@ const char *cli_get_last_virus(const cli_ctx *ctx);
 const char *cli_get_last_virus_str(const cli_ctx *ctx);
 void cli_virus_found_cb(cli_ctx *ctx);
 
-/**
- * @brief Push a new fmap onto our scan recursion stack.
- *
- * May fail if we exceed max recursion depth.
- *
- * @param ctx           The scanning context.
- * @param map           The fmap for the new layer.
- * @param type          The file type. May be CL_TYPE_ANY if unknown. Can change it later with cli_recursion_stack_change_type().
- * @param is_new_buffer true if the fmap represents a new buffer/file, and not some window into an existing fmap.
- * @return cl_error_t   CL_SUCCESS if successful, else CL_EMAXREC if exceeding the max recursion depth.
- */
-cl_error_t cli_recursion_stack_push(cli_ctx *ctx, cl_fmap_t *map, cli_file_t type, bool is_new_buffer);
-
-/**
- * @brief Pop off a layer of our scan recursion stack.
- *
- * Returns the fmap for the popped layer. Does NOT funmap() the fmap for you.
- *
- * @param ctx           The scanning context.
- * @return cl_fmap_t*   A pointer to the fmap for the popped layer, may return NULL instead if the stack is empty.
- */
-cl_fmap_t *cli_recursion_stack_pop(cli_ctx *ctx);
-
-/**
- * @brief Re-assign the type for the current layer.
- *
- * @param ctx   The scanning context.
- * @param type  The new file type.
- */
-void cli_recursion_stack_change_type(cli_ctx *ctx, cli_file_t type);
-
-/**
- * @brief Get the type of a specific layer.
- *
- * Ignores normalized layers internally.
- *
- * For index:
- *  0 == the outermost (bottom) layer of the stack.
- *  1 == the first layer (probably never explicitly used).
- * -1 == the present innermost (top) layer of the stack.
- * -2 == the parent layer (or "container"). That is, the second from the top of the stack.
- *
- * @param ctx           The scanning context.
- * @param index         Desired index, will be converted internally as though the normalized layers were stripped out. Don't think too had about it. Or do. ¯\_(ツ)_/¯
- * @return cli_file_t   The type of the requested layer,
- *                      or returns CL_TYPE_ANY if a negative layer is requested,
- *                      or returns CL_TYPE_IGNORED if requested layer too high.
- */
-cli_file_t cli_recursion_stack_get_type(cli_ctx *ctx, int index);
-
-/**
- * @brief Get the size of a specific layer.
- *
- * Ignores normalized layers internally.
- *
- * For index:
- *  0 == the outermost (bottom) layer of the stack.
- *  1 == the first layer (probably never explicitly used).
- * -1 == the present innermost (top) layer of the stack.
- * -2 == the parent layer (or "container"). That is, the second from the top of the stack.
- *
- * @param ctx           The scanning context.
- * @param index         Desired index, will be converted internally as though the normalized layers were stripped out. Don't think too had about it. Or do. ¯\_(ツ)_/¯
- * @return cli_file_t   The size of the requested layer,
- *                      or returns the size of the whole file if a negative layer is requested,
- *                      or returns 0 if requested layer too high.
- */
-size_t cli_recursion_stack_get_size(cli_ctx *ctx, int index);
+void cli_set_container(cli_ctx *ctx, cli_file_t type, size_t size);
+cli_file_t cli_get_container(cli_ctx *ctx, int index);
+size_t cli_get_container_size(cli_ctx *ctx, int index);
+cli_file_t cli_get_container_intermediate(cli_ctx *ctx, int index);
 
 /* used by: spin, yc (C) aCaB */
 #define __SHIFTBITS(a) (sizeof(a) << 3)
@@ -829,7 +755,7 @@ void cli_logg_unsetup(void);
 #define __hot__
 #endif
 
-#define cli_dbgmsg (!UNLIKELY(cli_debug_flag)) ? (void)0 : cli_dbgmsg_internal
+#define cli_dbgmsg (!UNLIKELY(cli_get_debug_flag())) ? (void)0 : cli_dbgmsg_internal
 
 #ifdef __GNUC__
 void cli_dbgmsg_internal(const char *str, ...) __attribute__((format(printf, 1, 2)));
@@ -848,6 +774,7 @@ static inline int cli_getpagesize(void)
     GetSystemInfo(&si);
     return si.dwPageSize;
 }
+#define HAVE_CLI_GETPAGESIZE 1
 #else /* ! _WIN32 */
 #if HAVE_SYSCONF_SC_PAGESIZE
 static inline int cli_getpagesize(void)
@@ -920,7 +847,7 @@ char *cli_newfilepath(const char *dir, const char *fname);
  * If the dir is not provided, the engine->tmpdir will be used.
  *
  * @param dir        Alternative temp directory (optional).
- * @param prefix  	 (Optional) Base filename for new file.
+ * @param fname  	 Filename for new file.
  * @param[out] name  Allocated filepath, must be freed by caller.
  * @param[out] fd    File descriptor of open temp file.
  */
@@ -932,6 +859,7 @@ cl_error_t cli_newfilepathfd(const char *dir, char *fname, char **name, int *fd)
  * Caller is responsible for freeing the filename.
  *
  * @param dir 	 Alternative temp directory. (optional)
+ * @param prefix (Optional) Prefix for new file tempfile.
  * @return char* filename or NULL.
  */
 char *cli_gentemp_with_prefix(const char *dir, const char *prefix);
@@ -965,7 +893,7 @@ cl_error_t cli_gentempfd(const char *dir, char **name, int *fd);
  * @param[out] fd    File descriptor of open temp file.
  * @return cl_error_t CL_SUCCESS, CL_ECREAT, or CL_EMEM.
  */
-cl_error_t cli_gentempfd_with_prefix(const char *dir, char *prefix, char **name, int *fd);
+cl_error_t cli_gentempfd_with_prefix(const char *dir, const char *prefix, char **name, int *fd);
 
 unsigned int cli_rndnum(unsigned int max);
 int cli_filecopy(const char *src, const char *dest);
@@ -974,21 +902,9 @@ void cli_bitset_free(bitset_t *bs);
 int cli_bitset_set(bitset_t *bs, unsigned long bit_offset);
 int cli_bitset_test(bitset_t *bs, unsigned long bit_offset);
 const char *cli_ctime(const time_t *timep, char *buf, const size_t bufsize);
-void cli_append_virus_if_heur_exceedsmax(cli_ctx *, char *);
+void cli_check_blockmax(cli_ctx *, int);
 cl_error_t cli_checklimits(const char *, cli_ctx *, unsigned long, unsigned long, unsigned long);
-
-/**
- * @brief Call before scanning a file to determine if we should scan it, skip it, or abort the entire scanning process.
- *
- * If the verdict is CL_SUCCESS, then this function increments the # of scanned files, and increments the amount of scanned data.
- * If the verdict is that a limit has been exceeded, then ctx->
- *
- * @param ctx       The scanning context.
- * @param needed    The size of the file we're considering scanning.
- * @return cl_error_t CL_SUCCESS if we're good to keep scanning else an error status.
- */
-cl_error_t cli_updatelimits(cli_ctx *ctx, size_t needed);
-
+cl_error_t cli_updatelimits(cli_ctx *, unsigned long);
 unsigned long cli_getsizelimit(cli_ctx *, unsigned long);
 int cli_matchregex(const char *str, const char *regex);
 void cli_qsort(void *a, size_t n, size_t es, int (*cmp)(const void *, const void *));
@@ -1077,6 +993,19 @@ cl_error_t cli_ftw(char *base, int flags, int maxdepth, cli_ftw_cb callback, str
 
 const char *cli_strerror(int errnum, char *buf, size_t len);
 
+#ifdef _WIN32
+/**
+ * @brief   Attempt to get a filename from an open file handle.
+ * 
+ * Windows only.
+ *
+ * @param hFile          File handle
+ * @param[out] filepath  Will be set to file path if found, or NULL.
+ * @return cl_error_t    CL_SUCCESS if found, else an error code.
+ */
+cl_error_t cli_get_filepath_from_handle(HANDLE hFile, char **filepath);
+#endif
+
 /**
  * @brief   Attempt to get a filename from an open file descriptor.
  *
@@ -1098,9 +1027,115 @@ cl_error_t cli_get_filepath_from_filedesc(int desc, char **filepath);
  * to get the real path.
  *
  * @param desc          A file path to evaluate.
- * @param char*         [out] A malloced string containing the real path.
+ * @param[out] char*    A malloced string containing the real path.
  * @return cl_error_t   CL_SUCCESS if found, else an error code.
  */
 cl_error_t cli_realpath(const char *file_name, char **real_filename);
 
+/**
+ * @brief   Get the libclamav debug flag (e.g. if debug logging is enabled)
+ *
+ * This is required for unit tests to be able to link with clamav.dll and not
+ * directly manipulate libclamav global variables.
+ */
+uint8_t cli_get_debug_flag();
+
+/**
+ * @brief   Set the libclamav debug flag to a specific value.
+ *
+ * The public cl_debug() API will only ever enable debug mode, it won't disable debug mode.
+ *
+ * This is required for unit tests to be able to link with clamav.dll and not
+ * directly manipulate libclamav global variables.
+ */
+uint8_t cli_set_debug_flag(uint8_t debug_flag);
+
+
+#ifndef STRDUP
+#define STRDUP(buf, var, ...)                  \
+    do {                                       \
+        var = strdup(buf);                     \
+        if (NULL == var) {                     \
+            do {                               \
+                __VA_ARGS__;                   \
+            } while (0);                       \
+            goto done;                         \
+        }                                      \
+    } while (0)
+#endif
+
+#ifndef FREE
+#define FREE(var)                              \
+    do {                                       \
+        if (NULL != var) {                     \
+            free(var);                         \
+            var = NULL;                        \
+        }                                      \
+    } while (0)
+#endif
+
+#ifndef MALLOC
+#define MALLOC(var, size, ...)                 \
+    do {                                       \
+        var = malloc(size);                    \
+        if (NULL == var) {                     \
+            do {                               \
+                __VA_ARGS__;                   \
+            } while (0);                       \
+            goto done;                         \
+        }                                      \
+    } while (0)
+#endif
+
+#ifndef CLI_MALLOC
+#define CLI_MALLOC(var, size, ...)             \
+    do {                                       \
+        var = cli_malloc(size);                \
+        if (NULL == var) {                     \
+            do {                               \
+                __VA_ARGS__;                   \
+            } while (0);                       \
+            goto done;                         \
+        }                                      \
+    } while (0)
+#endif
+
+#ifndef CALLOC
+#define CALLOC(var, nmemb, size, ...)       \
+    do {                                       \
+        (var) = calloc(nmemb, size);           \
+        if (NULL == var) {                     \
+            do {                               \
+                __VA_ARGS__;                   \
+            } while (0);                       \
+            goto done;                         \
+        }                                      \
+    } while (0)
+#endif
+
+#ifndef CLI_CALLOC
+#define CLI_CALLOC(var, nmemb, size, ...)      \
+    do {                                       \
+        (var) = cli_calloc(nmemb, size);       \
+        if (NULL == var) {                     \
+            do {                               \
+                __VA_ARGS__;                   \
+            } while (0);                       \
+            goto done;                         \
+        }                                      \
+    } while (0)
+#endif
+
+#ifndef VERIFY_POINTER
+#define VERIFY_POINTER(ptr, ...)            \
+    do {                                       \
+        if (NULL == ptr) {                     \
+            do {                               \
+                __VA_ARGS__;                   \
+            } while (0);                       \
+            goto done;                         \
+        }                                      \
+    } while (0)
+#endif
+
 #endif
