@@ -127,7 +127,7 @@ cl_error_t cli_parsepng(cli_ctx *ctx)
         status = CL_EARG;
         goto done;
     }
-    map = ctx->fmap;
+    map = *ctx->fmap;
 
     while (fmap_readn(map, (void *)&chunk_data_length_u32, offset, PNG_CHUNK_LENGTH_SIZE) == PNG_CHUNK_LENGTH_SIZE) {
         chunk_data_length = be32_to_host(chunk_data_length_u32);
@@ -346,7 +346,7 @@ cl_error_t cli_parsepng(cli_ctx *ctx)
                         cli_dbgmsg("PNG: zlib: inflate error: %d, Image decompression failed!\n", err);
                         inflateEnd(&zstrm);
                         zstrm_initialized = false;
-                        idat_state = PNG_IDAT_DECOMPRESSION_FAILED;
+                        idat_state        = PNG_IDAT_DECOMPRESSION_FAILED;
                         break;
                     }
                 }
@@ -355,7 +355,7 @@ cl_error_t cli_parsepng(cli_ctx *ctx)
                     cli_dbgmsg("  TOTAL decompressed:    %zu\n", decompressed_data_len);
                     inflateEnd(&zstrm);
                     zstrm_initialized = false;
-                    idat_state = PNG_IDAT_DECOMPRESSION_COMPLETE;
+                    idat_state        = PNG_IDAT_DECOMPRESSION_COMPLETE;
 
                     if ((decompressed_data_len > image_size) && (SCAN_HEURISTIC_BROKEN_MEDIA)) {
                         status = cli_append_virus(ctx, "Heuristics.PNG.CVE-2010-1205");
@@ -428,7 +428,7 @@ scan_overlay:
     }
 
     /* Check if there's an overlay, and scan it if one exists. */
-    if (map->len - offset > 0) {
+    if (map->len > offset) {
         cli_dbgmsg("PNG: Found " STDu64 " additional data after end of PNG! Scanning as a nested file.\n", map->len - offset);
         status = cli_magic_scan_nested_fmap_type(map, (size_t)offset, map->len - offset, ctx, CL_TYPE_ANY, NULL);
         goto done;
