@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Nigel Horne
@@ -159,7 +159,7 @@ int cli_tnef(const char *dir, cli_ctx *ctx)
 				 */
                 if (cli_debug_flag) {
                     int fout       = -1;
-                    char *filename = cli_gentemp(ctx->engine->tmpdir);
+                    char *filename = cli_gentemp(ctx->sub_tmpdir);
                     char buffer[BUFSIZ];
 
                     if (filename)
@@ -168,7 +168,7 @@ int cli_tnef(const char *dir, cli_ctx *ctx)
                     if (fout >= 0) {
                         size_t count;
 
-                        cli_warnmsg("Saving dump to %s:  refer to https://www.clamav.net/documents/installing-clamav\n", filename);
+                        cli_warnmsg("Saving dump to %s:  refer to https://docs.clamav.net/manual/Installing.html\n", filename);
 
                         pos = 0;
                         while ((count = fmap_readn(*ctx->fmap, buffer, pos, sizeof(buffer))) != (size_t)-1) {
@@ -266,7 +266,7 @@ tnef_message(fmap_t *map, off_t *pos, uint16_t type, uint16_t tag, int32_t lengt
 
     /*cli_dbgmsg("%lu %lu\n", (long)(offset + length), ftell(fp));*/
 
-    if (!CLI_ISCONTAINED2(0, fsize, offset, (off_t)length)) {
+    if (!CLI_ISCONTAINED2(0, fsize, offset, length)) {
         cli_dbgmsg("TNEF: Incorrect length field in tnef_message\n");
         return -1;
     }
