@@ -77,7 +77,7 @@
 #include "cvd.h"
 #include "regex_list.h"
 
-// shared
+// common
 #include "optparser.h"
 #include "output.h"
 #include "cdiff.h"
@@ -166,12 +166,12 @@ static void uuid_v4_gen(char *buffer)
 
 fc_error_t load_freshclam_dat(void)
 {
-    fc_error_t status      = FC_EINIT;
-    int handle             = -1;
-    ssize_t bread          = 0;
+    fc_error_t status        = FC_EINIT;
+    int handle               = -1;
+    ssize_t bread            = 0;
     freshclam_dat_v1_t *mdat = NULL;
-    uint32_t version       = 0;
-    char magic[13]         = {0};
+    uint32_t version         = 0;
+    char magic[13]           = {0};
 
     /* Change directory to database directory */
     if (chdir(g_databaseDirectory)) {
@@ -223,7 +223,12 @@ fc_error_t load_freshclam_dat(void)
             }
 
             /* Rewind to just after the magic bytes and read data struct */
-            lseek(handle, strlen(MIRRORS_DAT_MAGIC), SEEK_SET);
+            if (-1 == lseek(handle, strlen(MIRRORS_DAT_MAGIC), SEEK_SET)) {
+                char error_message[260];
+                cli_strerror(errno, error_message, 260);
+                logg("!Can't seek to %lu, error: %s\n", strlen(MIRRORS_DAT_MAGIC), error_message);
+                goto done;
+            }
 
             mdat = malloc(sizeof(freshclam_dat_v1_t));
             if (NULL == mdat) {
@@ -251,7 +256,7 @@ fc_error_t load_freshclam_dat(void)
                 free(g_freshclamDat);
             }
             g_freshclamDat = mdat;
-            mdat         = NULL;
+            mdat           = NULL;
             break;
         }
         default: {
@@ -792,7 +797,7 @@ static size_t WriteFileCallback(void *contents, size_t size, size_t nmemb, void
  * @param ifModifiedSince   modified time of local database. May be 0 to always get the CVD header.
  * @param server            server to use to retrieve for database header.
  * @param logerr            non-zero to upgrade warnings to errors.
- * @param cvd               [out] CVD header of newest available CVD, if FC_SUCCESS
+ * @param[out] cvd          CVD header of newest available CVD, if FC_SUCCESS
  * @return fc_error_t       FC_SUCCESS if CVD header obtained.
  * @return fc_error_t       FC_UPTODATE if received 304 in response to ifModifiedSince date.
  * @return fc_error_t       Another error code if failure occured.
@@ -877,25 +882,25 @@ static fc_error_t remote_cvdhead(
        if both callbacks are set. */
 
         if (CURLE_OK != curl_easy_setopt(curl, CURLOPT_XFERINFOFUNCTION, xferinfo)) {
-            logg("!create_curl_handle: Failed to set transfer info function!\n");
+            logg("!remote_cvdhead: Failed to set transfer info function!\n");
         }
         /* pass the struct pointer into the xferinfo function, note that this is
            an alias to CURLOPT_PROGRESSDATA */
         if (CURLE_OK != curl_easy_setopt(curl, CURLOPT_XFERINFODATA, &prog)) {
-            logg("!create_curl_handle: Failed to set transfer info data structure!\n");
+            logg("!remote_cvdhead: Failed to set transfer info data structure!\n");
         }
 #else
         if (CURLE_OK != curl_easy_setopt(curl, CURLOPT_PROGRESSFUNCTION, older_progress)) {
-            logg("!create_curl_handle: Failed to set progress function!\n");
+            logg("!remote_cvdhead: Failed to set progress function!\n");
         }
         /* pass the struct pointer into the progress function */
         if (CURLE_OK != curl_easy_setopt(curl, CURLOPT_PROGRESSDATA, &prog)) {
-            logg("!create_curl_handle: Failed to set progress data structure!\n");
+            logg("!remote_cvdhead: Failed to set progress data structure!\n");
         }
 #endif
 
         if (CURLE_OK != curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 0L)) {
-            logg("!create_curl_handle: Failed to disable progress function!\n");
+            logg("!remote_cvdhead: Failed to disable progress function!\n");
         }
     }
 
@@ -997,12 +1002,6 @@ static fc_error_t remote_cvdhead(
         }
         case 403: {
             status = FC_EFORBIDDEN;
-
-            /* Try again in no less than 24 hours if freshclam received a 403 FORBIDDEN. */
-            g_freshclamDat->retry_after = time(NULL) + 60 * 60 * 24;
-
-            (void)save_freshclam_dat();
-
             break;
         }
         case 429: {
@@ -1290,11 +1289,7 @@ static fc_error_t downloadFile(
     switch (http_code) {
         case 200:
         case 206: {
-            if (0 == receivedFile.size) {
-                status = FC_EEMPTYFILE;
-            } else {
-                status = FC_SUCCESS;
-            }
+            status = FC_SUCCESS;
             break;
         }
         case 304: {
@@ -1303,12 +1298,6 @@ static fc_error_t downloadFile(
         }
         case 403: {
             status = FC_EFORBIDDEN;
-
-            /* Try again in no less than 24 hours if freshclam received a 403 FORBIDDEN. */
-            g_freshclamDat->retry_after = time(NULL) + 60 * 60 * 24;
-
-            (void)save_freshclam_dat();
-
             break;
         }
         case 429: {
@@ -1490,7 +1479,7 @@ done:
  * Will create the temp dir if it does not already exist.
  *
  * @param database      The database we're updating.
- * @param tmpdir        [out] The name of the temp dir to use.
+ * @param[out] tmpdir   The name of the temp dir to use.
  * @return fc_error_t
  */
 static fc_error_t mkdir_and_chdir_for_cdiff_tmp(const char *database, const char *tmpdir)
@@ -1661,7 +1650,7 @@ done:
  * @brief Get CVD header info for local CVD/CLD database.
  *
  * @param database          Database name
- * @param localname         [out] (optional) filename of local database.
+ * @param[out] localname    (optional) filename of local database.
  * @return struct cl_cvd*   CVD info struct of local database, if found. NULL if not found.
  */
 static struct cl_cvd *currentdb(const char *database, char **localname)
@@ -1810,7 +1799,7 @@ static fc_error_t buildcld(
         }
     }
 
-    if (NULL == (dir = opendir("."))) {
+    if (NULL == (dir = opendir(tmpdir))) {
         logg("!buildcld: Can't open directory %s\n", tmpdir);
         status = FC_EDIRECTORY;
         goto done;
@@ -2361,6 +2350,7 @@ fc_error_t updatedb(
             newLocalFilename = cli_strdup(remoteFilename);
         } else if (0 == numPatchesReceived) {
             logg("The database server doesn't have the latest patch for the %s database (version %u). The server will likely have updated if you check again in a few hours.\n", database, remoteVersion);
+            *dbFilename = cli_strdup(localFilename);
             goto up_to_date;
         } else {
             /*
@@ -2423,8 +2413,8 @@ fc_error_t updatedb(
      */
 #ifdef _WIN32
     if (!access(newLocalFilename, R_OK) && unlink(newLocalFilename)) {
-        logg("!Update failed. Can't delete the old database %s to replace it with a new database. Please fix the problem manually and try again.\n", newLocalFilename);
-        status = FC_EDBDIRACCESS;
+        logg("!updatedb: Can't delete old database %s. Please fix the problem manually and try again.\n", newLocalFilename);
+        status = FC_EEMPTYFILE;
         goto done;
     }
 #endif
@@ -2610,7 +2600,7 @@ fc_error_t updatecustomdb(
         }
         snprintf(tmpfile_with_extension, tmpfile_with_extension_len + 1, "%s-%s", tmpfile, databaseName);
         if (rename(tmpfile, tmpfile_with_extension) == -1) {
-            logg("!Custom database update failed: Can't rename %s to %s: %s\n", tmpfile, tmpfile_with_extension, strerror(errno));
+            logg("!updatecustomdb: Can't rename %s to %s: %s\n", tmpfile, tmpfile_with_extension, strerror(errno));
             free(tmpfile_with_extension);
             status = FC_EDBDIRACCESS;
             goto done;
@@ -2633,8 +2623,8 @@ fc_error_t updatecustomdb(
      */
 #ifdef _WIN32
     if (!access(databaseName, R_OK) && unlink(databaseName)) {
-        logg("!Custom database update failed. Can't delete the old database %s to replace it with a new database. Please fix the problem manually and try again.\n", databaseName);
-        status = FC_EDBDIRACCESS;
+        logg("!updatecustomdb: Can't delete old database %s. Please fix the problem manually and try again.\n", databaseName);
+        status = FC_EEMPTYFILE;
         goto done;
     }
 #endif
